/*
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.network.synchronization.protocol;

import com.topcoder.network.synchronization.bean.*;
import com.topcoder.network.synchronization.exception.*;

/**
 * This interface defines a peer group service.
 * A peer group service needs a message sender, message receiver, message listeners, etc.
 * It privodes the functionality to send messages to the peer group and receive messages from
 * the peer group.  
 * 
 * 
 * @author TCSDESIGNER
 * @version 1.0
 */
public abstract class PeerGroupService implements Runnable {   
    
    /**
     * Represents whether the peer group service is started.
     */
    protected boolean started = false;
    
    /**
     * Represents the peer group in this service.
     */
    protected PeerGroupInfo groupInfo = null;
    
    /**
     * Pepresents the peer info in this service.
     */
    protected PeerInfo peerInfo = null;
    
    /**
     * Represents the message sender in this service.
     */
    protected MessageSender sender = null;
    
    /**
     * Represents the message receiver in this service.
     */
    protected MessageReceiver receiver = null;
    
    /**
     * Represnets the first APPLICATION_MESSAGE listener in this service.
     */
    protected MessageListener firstAppListener = null;
    
    /**
     * Represents the last APPLICATION_MESSAGE listener in this service.
     */
    protected MessageListener lastAppListener = null;
    
    /**
     * Represents the error handling listener in this service.
     */
    protected ErrorHandlingListener errorListener = null;
    
    /**
     * This abstract run method should be implemented in the subclass.
     * The business logic of this service is in this method. 
     */
    public abstract void run();
    
    /**
     * Stop the service.
     * This method set the started flag to false, stop the sender and receiver.
     *
     */
    public void stop() {
        
    }
    
    /**
     * Set the peer group info of this service.
     * 
     * @param groupInfo the peer group info of this service
     */
    public void setPeerGroupInfo(PeerGroupInfo groupInfo) {
        
    }
    
    /**
     * Return the peer group info of this service.
     * 
     * @return the peer group info of this service
     */
    public PeerGroupInfo getPeerGroupInfo() {
        return null;
    }
    
    /**
     * Set the peer info of this service.
     * 
     * @param peerInfo the peer info of this service
     */
    public void setPeerInfo(PeerInfo peerInfo) {
        
    }
    
    /**
     * Return the peer info of this service.
     * @return the peer info of this service.
     */
    public PeerInfo getPeerInfo() {
        return null;
    }
    
    /**
     * set the message sender of this service.
     * @param sender the message sender of this service
     */
    public void setMessageSender(MessageSender sender) {
        
    }
    
    /**
     * Return the message sender of this service.
     * @return the message sender of this service
     */
    public MessageSender getMessageSender() {
        return null;
    }
    
    /**
     * Set the message receiver of this service.
     * @param receiver the message receiver of this service
     */
    public void setMessageReceiver(MessageReceiver receiver) {
        
    }
    
    /**
     * Return the message receiver of this service.
     * @return the message receiver of this service
     */
    public MessageReceiver getMessageReceiver() {
        return null;
    }
    
    /**
     * Return whether the service is started.
     * 
     * @return whether the service is started
     */
    public boolean isStarted() {
        return false;
    }
    
    /**
     * Send ths message to the peer group, all the peers of this peer group will receive ths message.
     * Simply call message sender to send this message.
     * 
     * @param message the message to send
     * @throws DistributedProtocolException if fails to send the message
     * @throws IllegalStateException if the sender is not sended
     */
    public void send(Message message) throws DistributedProtocolException {
        
    }
    
    /**
     * Add APPLICATION_MESSAGE listener to this service.
     * 
     * @param listener APPLICATION_MESSAGE listener to add
     */
    public void addApplicationMessageListener(MessageListener listener) {
        
    }
    
    /**
     * Remove APPLICATION_MESSAGE listener to this service.
     * @param listener the APPLICATION_MESSAGE listener to remove
     */
    public void removeApplicationMessageListener(MessageListener listener) {
        
    }
    
    /**
     * Return the first APPLICATION_MESSAGE listener in this service.
     * @return the first APPLICATION_MESSAGE listener in this service.
     */
    public MessageListener getApplicationMessageListener() {
        return null;
    }
    
    /**
     * Set the error handling listener to this service
     * 
     * @param errorListener the error handling listener to handle exceptions.
     */
    public void setErrorHandlingListener(ErrorHandlingListener errorListener) {
        
    }
    
    /**
     * Return the error handling listener to handle exceptions
     * 
     * @return the error handling listener to handle exceptions
     */
    public ErrorHandlingListener getErrorHandlingListener() {
        return null;
    }

}
